@@ -37,6 +37,32 @@
 #include <assert.h>
 #endif
 
+static struct tevent_context *
+test_tevent_context_init(TALLOC_CTX *mem_ctx)
+{
+	struct tevent_context *ev = NULL;
+
+	ev = tevent_context_init(mem_ctx);
+	if (ev != NULL) {
+		samba_tevent_set_debug(ev, "<default>");
+	}
+
+	return ev;
+}
+
+static struct tevent_context *
+test_tevent_context_init_byname(TALLOC_CTX *mem_ctx, const char *name)
+{
+	struct tevent_context *ev = NULL;
+
+	ev = tevent_context_init_byname(mem_ctx, name);
+	if (ev != NULL) {
+		samba_tevent_set_debug(ev, name);
+	}
+
+	return ev;
+}
+
 static int fde_count;
 
 static void do_read(int fd, void *buf, size_t count)
@@ -143,7 +169,7 @@ static bool test_event_context(struct torture_context *test,
 	struct timeval t;
 	int ret;
 
-	ev_ctx = tevent_context_init_byname(test, backend);
+	ev_ctx = test_tevent_context_init_byname(test, backend);
 	if (ev_ctx == NULL) {
 		torture_comment(test, "event backend '%s' not supported\n", backend);
 		return true;
@@ -383,7 +409,7 @@ static bool test_event_fd1(struct torture_context *tctx,
 	state.tctx = tctx;
 	state.backend = (const char *)test_data;
 
-	state.ev = tevent_context_init_byname(tctx, state.backend);
+	state.ev = test_tevent_context_init_byname(tctx, state.backend);
 	if (state.ev == NULL) {
 		torture_skip(tctx, talloc_asprintf(tctx,
 			     "event backend '%s' not supported\n",
@@ -391,7 +417,6 @@ static bool test_event_fd1(struct torture_context *tctx,
 		return true;
 	}
 
-	tevent_set_debug_stderr(state.ev);
 	torture_comment(tctx, "backend '%s' - %s\n",
 			state.backend, __FUNCTION__);
 
@@ -423,7 +448,7 @@ static bool test_event_fd1(struct torture_context *tctx,
 	torture_assert(tctx, ret == 0, "socketpair() failed");
 
 	state.te = tevent_add_timer(state.ev, state.ev,
-				    timeval_current_ofs(0,1000),
+				    timeval_current_ofs(0,10000),
 				    test_event_fd1_finished, &state);
 	state.fde0 = tevent_add_fd(state.ev, state.ev,
 				   state.sock[0], TEVENT_FD_WRITE,
@@ -623,7 +648,7 @@ static bool test_event_fd2(struct torture_context *tctx,
 	state.tctx = tctx;
 	state.backend = (const char *)test_data;
 
-	state.ev = tevent_context_init_byname(tctx, state.backend);
+	state.ev = test_tevent_context_init_byname(tctx, state.backend);
 	if (state.ev == NULL) {
 		torture_skip(tctx, talloc_asprintf(tctx,
 			     "event backend '%s' not supported\n",
@@ -631,7 +656,6 @@ static bool test_event_fd2(struct torture_context *tctx,
 		return true;
 	}
 
-	tevent_set_debug_stderr(state.ev);
 	torture_comment(tctx, "backend '%s' - %s\n",
 			state.backend, __FUNCTION__);
 
@@ -956,7 +980,7 @@ static bool test_wrapper(struct torture_context *tctx,
 	bool ok = false;
 	bool ret2;
 
-	ev = tevent_context_init_byname(tctx, backend);
+	ev = test_tevent_context_init_byname(tctx, backend);
 	if (ev == NULL) {
 		torture_skip(tctx, talloc_asprintf(tctx,
 			     "event backend '%s' not supported\n",
@@ -964,7 +988,6 @@ static bool test_wrapper(struct torture_context *tctx,
 		return true;
 	}
 
-	tevent_set_debug_stderr(ev);
 	torture_comment(tctx, "tevent backend '%s'\n", backend);
 
 	wrap_ev = tevent_context_wrapper_create(
@@ -1130,7 +1153,7 @@ static bool test_free_wrapper(struct torture_context *tctx,
 	int ret;
 	bool ok = false;
 
-	ev = tevent_context_init_byname(frame, backend);
+	ev = test_tevent_context_init_byname(frame, backend);
 	if (ev == NULL) {
 		torture_skip(tctx, talloc_asprintf(tctx,
 			     "event backend '%s' not supported\n",
@@ -1138,7 +1161,6 @@ static bool test_free_wrapper(struct torture_context *tctx,
 		return true;
 	}
 
-	tevent_set_debug_stderr(ev);
 	torture_comment(tctx, "tevent backend '%s'\n", backend);
 
 	wrap_ev = tevent_context_wrapper_create(
@@ -1294,7 +1316,7 @@ static bool test_event_context_threaded(struct torture_context *test,
 	int ret;
 	char c = 0;
 
-	ev = tevent_context_init_byname(test, "poll_mt");
+	ev = test_tevent_context_init_byname(test, "poll_mt");
 	torture_assert(test, ev != NULL, "poll_mt not supported");
 
 	tevent_set_trace_callback(ev, test_event_threaded_trace, NULL);
@@ -1411,11 +1433,10 @@ static bool test_multi_tevent_threaded(struct torture_context *test,
 	thread_test_ctx = test;
 	thread_counter = 0;
 
-	master_ev = tevent_context_init(NULL);
+	master_ev = test_tevent_context_init(NULL);
 	if (master_ev == NULL) {
 		return false;
 	}
-	tevent_set_debug_stderr(master_ev);
 
 	tp = tevent_thread_proxy_create(master_ev);
 	if (tp == NULL) {
@@ -1612,11 +1633,10 @@ static bool test_multi_tevent_threaded_1(struct torture_context *test,
 	thread_test_ctx = test;
 	thread_counter = 0;
 
-	master_ev = tevent_context_init(NULL);
+	master_ev = test_tevent_context_init(NULL);
 	if (master_ev == NULL) {
 		return false;
 	}
-	tevent_set_debug_stderr(master_ev);
 
 	master_tp = tevent_thread_proxy_create(master_ev);
 	if (master_tp == NULL) {
@@ -1714,7 +1734,7 @@ static bool test_multi_tevent_threaded_2(struct torture_context *test,
 	thread_test_ctx = test;
 	thread_counter = 0;
 
-	ev = tevent_context_init(test);
+	ev = test_tevent_context_init(test);
 	torture_assert(test, ev != NULL, "tevent_context_init failed");
 
 	/*
@@ -1761,8 +1781,75 @@ static bool test_multi_tevent_threaded_2(struct torture_context *test,
 	talloc_free(ev);
 	return true;
 }
+
+struct test_cached_pid_thread_state {
+	pid_t thread_cached_pid;
+	pid_t thread_pid;
+};
+
+static void *test_cached_pid_thread(void *private_data)
+{
+	struct test_cached_pid_thread_state *state =
+		(struct test_cached_pid_thread_state *)private_data;
+
+	state->thread_cached_pid = tevent_cached_getpid();
+	state->thread_pid = getpid();
+
+	return NULL;
+}
 #endif
 
+static bool test_cached_pid(struct torture_context *test,
+			    const void *test_data)
+{
+	pid_t parent_pid = getpid();
+	pid_t child_pid;
+	pid_t finished_pid;
+	int child_status;
+
+	torture_assert(test, tevent_cached_getpid() == parent_pid, "tevent_cached_getpid()");
+
+#ifdef HAVE_PTHREAD
+	{
+		struct test_cached_pid_thread_state state = { .thread_cached_pid = -1, };
+		pthread_t thread;
+		void *retval = NULL;
+		int ret;
+
+		ret = pthread_create(&thread, NULL, test_cached_pid_thread, &state);
+		torture_assert(test, ret == 0, "pthread_create failed");
+
+		ret = pthread_join(thread, &retval);
+		torture_assert(test, ret == 0, "pthread_join failed");
+
+		torture_assert(test, state.thread_pid == parent_pid, "getpid() in thread");
+		torture_assert(test, state.thread_cached_pid == parent_pid, "tevent_cached_getpid() in thread");
+	}
+#endif /* HAVE_PTHREAD */
+
+	child_pid = fork();
+	if (child_pid == 0) {
+		/* child */
+		pid_t pid = getpid();
+		pid_t cached_pid = tevent_cached_getpid();
+
+		if (parent_pid == pid) {
+			exit(1);
+		}
+		if (pid != cached_pid) {
+			exit(2);
+		}
+		exit(0);
+	}
+	torture_assert(test, child_pid > 0, "fork failed");
+
+	finished_pid = waitpid(child_pid, &child_status, 0);
+	torture_assert(test, finished_pid == child_pid, "wrong child");
+	torture_assert(test, child_status == 0, "child_status");
+
+	return true;
+}
+
 struct torture_suite *torture_local_event(TALLOC_CTX *mem_ctx)
 {
 	struct torture_suite *suite = torture_suite_create(mem_ctx, "event");
@@ -1817,5 +1904,9 @@ struct torture_suite *torture_local_event(TALLOC_CTX *mem_ctx)
 
 #endif
 
+	torture_suite_add_simple_tcase_const(suite, "tevent_cached_getpid",
+					     test_cached_pid,
+					     NULL);
+
 	return suite;
 }
