@@ -1,4 +1,4 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    main select loop and event handling
    Copyright (C) Andrew Tridgell 2003
@@ -130,14 +130,12 @@ static void tevent_backend_init(void)
 	tevent_poll_mt_init();
 #if defined(HAVE_EPOLL)
 	tevent_epoll_init();
-#elif defined(HAVE_SOLARIS_PORTS)
-	tevent_port_init();
 #endif
 
 	tevent_standard_init();
 }
 
-_PRIVATE_ const struct tevent_ops *tevent_find_ops_byname(const char *name)
+const struct tevent_ops *tevent_find_ops_byname(const char *name)
 {
 	struct tevent_ops_list *e;
 
@@ -166,14 +164,28 @@ const char **tevent_backend_list(TALLOC_CTX *mem_ctx)
 {
 	const char **list = NULL;
 	struct tevent_ops_list *e;
+	size_t idx = 0;
 
 	tevent_backend_init();
 
 	for (e=tevent_backends;e;e=e->next) {
-		list = ev_str_list_add(list, e->name);
+		idx += 1;
 	}
 
-	talloc_steal(mem_ctx, list);
+	list = talloc_zero_array(mem_ctx, const char *, idx+1);
+	if (list == NULL) {
+		return NULL;
+	}
+
+	idx = 0;
+	for (e=tevent_backends;e;e=e->next) {
+		list[idx] = talloc_strdup(list, e->name);
+		if (list[idx] == NULL) {
+			TALLOC_FREE(list);
+			return NULL;
+		}
+		idx += 1;
+	}
 
 	return list;
 }
@@ -185,6 +197,7 @@ static void tevent_common_wakeup_fini(struct tevent_context *ev);
 static pthread_mutex_t tevent_contexts_mutex = PTHREAD_MUTEX_INITIALIZER;
 static struct tevent_context *tevent_contexts = NULL;
 static pthread_once_t tevent_atfork_initialized = PTHREAD_ONCE_INIT;
+static pid_t tevent_cached_global_pid = 0;
 
 static void tevent_atfork_prepare(void)
 {
@@ -249,6 +262,8 @@ static void tevent_atfork_child(void)
 	struct tevent_context *ev;
 	int ret;
 
+	tevent_cached_global_pid = getpid();
+
 	for (ev = DLIST_TAIL(tevent_contexts); ev != NULL;
 	     ev = DLIST_PREV(ev)) {
 		struct tevent_threaded_context *tctx;
@@ -288,10 +303,42 @@ static void tevent_prep_atfork(void)
 	if (ret != 0) {
 		abort();
 	}
+
+	tevent_cached_global_pid = getpid();
 }
 
 #endif
 
+static int tevent_init_globals(void)
+{
+#ifdef HAVE_PTHREAD
+	int ret;
+
+	ret = pthread_once(&tevent_atfork_initialized, tevent_prep_atfork);
+	if (ret != 0) {
+		return ret;
+	}
+#endif
+
+	return 0;
+}
+
+_PUBLIC_ pid_t tevent_cached_getpid(void)
+{
+#ifdef HAVE_PTHREAD
+	tevent_init_globals();
+#ifdef TEVENT_VERIFY_CACHED_GETPID
+	if (tevent_cached_global_pid != getpid()) {
+		tevent_abort(NULL, "tevent_cached_global_pid invalid");
+	}
+#endif
+	if (tevent_cached_global_pid != 0) {
+		return tevent_cached_global_pid;
+	}
+#endif
+	return getpid();
+}
+
 int tevent_common_context_destructor(struct tevent_context *ev)
 {
 	struct tevent_fd *fd, *fn;
@@ -363,6 +410,7 @@ int tevent_common_context_destructor(struct tevent_context *ev)
 
 	for (fd = ev->fd_events; fd; fd = fn) {
 		fn = fd->next;
+		tevent_trace_fd_callback(fd->event_ctx, fd, TEVENT_EVENT_TRACE_DETACH);
 		fd->wrapper = NULL;
 		fd->event_ctx = NULL;
 		DLIST_REMOVE(ev->fd_events, fd);
@@ -371,6 +419,7 @@ int tevent_common_context_destructor(struct tevent_context *ev)
 	ev->last_zero_timer = NULL;
 	for (te = ev->timer_events; te; te = tn) {
 		tn = te->next;
+		tevent_trace_timer_callback(te->event_ctx, te, TEVENT_EVENT_TRACE_DETACH);
 		te->wrapper = NULL;
 		te->event_ctx = NULL;
 		DLIST_REMOVE(ev->timer_events, te);
@@ -378,6 +427,7 @@ int tevent_common_context_destructor(struct tevent_context *ev)
 
 	for (ie = ev->immediate_events; ie; ie = in) {
 		in = ie->next;
+		tevent_trace_immediate_callback(ie->event_ctx, ie, TEVENT_EVENT_TRACE_DETACH);
 		ie->wrapper = NULL;
 		ie->event_ctx = NULL;
 		ie->cancel_fn = NULL;
@@ -386,6 +436,7 @@ int tevent_common_context_destructor(struct tevent_context *ev)
 
 	for (se = ev->signal_events; se; se = sn) {
 		sn = se->next;
+		tevent_trace_signal_callback(se->event_ctx, se, TEVENT_EVENT_TRACE_DETACH);
 		se->wrapper = NULL;
 		se->event_ctx = NULL;
 		DLIST_REMOVE(ev->signal_events, se);
@@ -416,13 +467,13 @@ static int tevent_common_context_constructor(struct tevent_context *ev)
 {
 	int ret;
 
-#ifdef HAVE_PTHREAD
-
-	ret = pthread_once(&tevent_atfork_initialized, tevent_prep_atfork);
+	ret = tevent_init_globals();
 	if (ret != 0) {
 		return ret;
 	}
 
+#ifdef HAVE_PTHREAD
+
 	ret = pthread_mutex_init(&ev->scheduled_mutex, NULL);
 	if (ret != 0) {
 		return ret;
@@ -710,7 +761,7 @@ void tevent_loop_set_nesting_hook(struct tevent_context *ev,
 				  tevent_nesting_hook hook,
 				  void *private_data)
 {
-	if (ev->nesting.hook_fn && 
+	if (ev->nesting.hook_fn &&
 	    (ev->nesting.hook_fn != hook ||
 	     ev->nesting.hook_private != private_data)) {
 		/* the way the nesting hook code is currently written
@@ -736,7 +787,7 @@ static void tevent_abort_nesting(struct tevent_context *ev, const char *location
 }
 
 /*
-  do a single event loop using the events defined in ev 
+  do a single event loop using the events defined in ev
 */
 int _tevent_loop_once(struct tevent_context *ev, const char *location)
 {
@@ -772,6 +823,9 @@ int _tevent_loop_once(struct tevent_context *ev, const char *location)
 	ret = ev->ops->loop_once(ev, location);
 	tevent_trace_point_callback(ev, TEVENT_TRACE_AFTER_LOOP_ONCE);
 
+	/* New event (and request) will always start with call depth 0. */
+	tevent_thread_call_depth_set(0);
+
 	if (ev->nesting.level > 0) {
 		if (ev->nesting.hook_fn) {
 			int ret2;
@@ -918,7 +972,7 @@ int _tevent_loop_wait(struct tevent_context *ev, const char *location)
 /*
   re-initialise a tevent context. This leaves you with the same
   event context, but all events are wiped and the structure is
-  re-initialised. This is most useful after a fork()  
+  re-initialised. This is most useful after a fork()
 
   zero is returned on success, non-zero on failure
 */
